"use strict";
/********************************************************************************
 * Copyright (C) 2021 Jon Radchenko and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is available at
 * https://www.gnu.org/software/classpath/license.html.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 ********************************************************************************/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RStudioOpenHandler = void 0;
const core_1 = require("@theia/core");
const inversify_1 = require("inversify");
const browser_1 = require("@theia/core/lib/browser");
const server_proxy_instance_manager_1 = require("theia-server-proxy-extension/lib/browser/server-proxy-instance-manager");
const workspace_service_1 = require("@theia/workspace/lib/browser/workspace-service");
const const_1 = require("../common/const");
const server_proxy_open_handler_1 = require("theia-server-proxy-extension/lib/browser/server-proxy-open-handler");
const server_proxy_url_manager_1 = require("theia-server-proxy-extension/lib/common/server-proxy-url-manager");
let RStudioOpenHandler = class RStudioOpenHandler {
    constructor() {
        this.id = 'rstudio.openhandler';
        this.label = 'Open in RStudio';
    }
    canHandle(uri) {
        try {
            const lowercaseExtension = uri.path.ext.toLocaleLowerCase();
            if (lowercaseExtension == ".rmd" || lowercaseExtension == ".r") {
                return 500;
            }
            return 0;
        }
        catch (_a) {
            return 0;
        }
    }
    async fallback(uri) {
        const openHandler = (await this.openerService.getOpeners(uri))
            .find(p => p != this);
        return openHandler === null || openHandler === void 0 ? void 0 : openHandler.open(uri);
    }
    async open(uri) {
        // todo handle nonworkspace file
        const workspace = this.workspaceService.getWorkspaceRootUri(uri);
        if (!workspace) {
            return this.fallback(uri);
        }
        const relativeFilePath = workspace.relative(uri);
        if (!relativeFilePath) {
            return this.fallback(uri);
        }
        const instance = await this.serverProxyInstanceManager.getOrCreateInstance(const_1.Extension.ServerProxy, {});
        if (!instance) {
            this.logger.warn("RStudio instance did not start properly");
            return this.fallback(uri);
        }
        const widget = await this.serverProxyOpenHandler.openInstance(instance);
        if (!widget) {
            this.logger.warn("RStudio widget did not open");
            return this.fallback(uri);
        }
        const data = JSON.stringify({
            method: "navigate_to_file",
            params: {
                file: uri.path.toString()
            }
        });
        const url = this.serverProxyUrlManager.getPublicPath(const_1.Extension.ServerProxy, instance.id, "rpc/theia_remote");
        await fetch(url, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Content-Length': data.length.toString(),
            },
            body: data
        });
    }
};
__decorate([
    (0, inversify_1.inject)(workspace_service_1.WorkspaceService),
    __metadata("design:type", workspace_service_1.WorkspaceService)
], RStudioOpenHandler.prototype, "workspaceService", void 0);
__decorate([
    (0, inversify_1.inject)(server_proxy_instance_manager_1.ServerProxyInstanceManager),
    __metadata("design:type", server_proxy_instance_manager_1.ServerProxyInstanceManager)
], RStudioOpenHandler.prototype, "serverProxyInstanceManager", void 0);
__decorate([
    (0, inversify_1.inject)(server_proxy_open_handler_1.ServerProxyOpenHandler),
    __metadata("design:type", server_proxy_open_handler_1.ServerProxyOpenHandler)
], RStudioOpenHandler.prototype, "serverProxyOpenHandler", void 0);
__decorate([
    (0, inversify_1.inject)(server_proxy_url_manager_1.ServerProxyUrlManager),
    __metadata("design:type", server_proxy_url_manager_1.ServerProxyUrlManager)
], RStudioOpenHandler.prototype, "serverProxyUrlManager", void 0);
__decorate([
    (0, inversify_1.inject)(browser_1.OpenerService),
    __metadata("design:type", Object)
], RStudioOpenHandler.prototype, "openerService", void 0);
__decorate([
    (0, inversify_1.inject)(core_1.ILogger),
    __metadata("design:type", Object)
], RStudioOpenHandler.prototype, "logger", void 0);
RStudioOpenHandler = __decorate([
    (0, inversify_1.injectable)()
], RStudioOpenHandler);
exports.RStudioOpenHandler = RStudioOpenHandler;
//# sourceMappingURL=rstudio-open-handler.js.map