"use strict";
/********************************************************************************
 * Copyright (C) 2021 Jon Radchenko and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is available at
 * https://www.gnu.org/software/classpath/license.html.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 ********************************************************************************/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RStudioServerProxyContribution = exports.RStudioServerProxyInstanceBuilder = void 0;
const http_proxy_middleware_1 = require("http-proxy-middleware");
const inversify_1 = require("inversify");
const server_proxy_contribution_1 = require("theia-server-proxy-extension/lib/node/server-proxy-contribution");
const server_proxy_1 = require("theia-server-proxy-extension/lib/common/server-proxy");
const os = require("os");
const const_1 = require("../common/const");
const server_proxy_url_manager_1 = require("theia-server-proxy-extension/lib/common/server-proxy-url-manager");
const core_1 = require("@theia/core");
const rstudio_cli_contribution_1 = require("./rstudio-cli-contribution");
let RStudioServerProxyInstanceBuilder = class RStudioServerProxyInstanceBuilder extends server_proxy_contribution_1.BaseServerProxyInstanceBuilder {
    constructor() {
        super(...arguments);
        this.serverProxy = const_1.Extension.ServerProxy;
    }
    async build(instanceId, context) {
        if (!this.instance || server_proxy_1.ServerProxyInstanceStatus.isCompleted(this.instance.status)) {
            this.instance = await super.build(instanceId, context);
        }
        return this.instance;
    }
    async getCommand(instanceId, context) {
        // TODO configurable
        const port = 8787;
        const command = [
            '/usr/lib/rstudio-server/bin/rserver',
            '--server-daemonize=0',
            '--auth-none=1',
            '--www-frame-origin=any',
            `--www-port=${port.toString()}`,
            '--auth-minimum-user-id=0' //allow root
        ];
        return {
            port,
            command,
            env: {
                "USER": os.userInfo().username
            }
        };
    }
};
RStudioServerProxyInstanceBuilder = __decorate([
    (0, inversify_1.injectable)()
], RStudioServerProxyInstanceBuilder);
exports.RStudioServerProxyInstanceBuilder = RStudioServerProxyInstanceBuilder;
let RStudioServerProxyContribution = class RStudioServerProxyContribution {
    constructor(serverProxyInstanceBuilder) {
        this.serverProxyInstanceBuilder = serverProxyInstanceBuilder;
        this.id = "rstudio";
        this.name = "RStudio";
    }
    getMiddleware(basePath, baseOptions) {
        const pathRewriter = {};
        pathRewriter[this.serverProxyUrlManager.getPathMatchRegex()] = '';
        baseOptions.pathRewrite = pathRewriter;
        baseOptions.onProxyRes = (proxyRes, req, res) => {
            var redirect = proxyRes.headers.location;
            if (!redirect) {
                return;
            }
            const serverProxyBasePath = req.serverProxyBasePath;
            const hostname = req.headers["host"];
            // TODO https
            // this should probably be changed altogether to not depend on the user's request
            redirect = redirect.replace('http://localhost:8787/', `http://${hostname}${this.serverProxyUrlManager.toPublicPath(serverProxyBasePath)}`);
            console.log(`Settings redirect from '${proxyRes.headers.location}' to '${redirect}'`);
            proxyRes.headers.location = redirect;
        };
        const baseOnProxyReq = baseOptions.onProxyReq;
        baseOptions.onProxyReq = (proxyReq, request, response) => {
            var _a;
            if (request.url == "/rpc/set_user_state" || request.url == "/rpc/set_client_state") {
                const json = request.body;
                this.clientId = json.clientId;
                this.clientVersion = json.clientVersion;
                this.csrfToken = (_a = request.headers["x-csrf-token"]) === null || _a === void 0 ? void 0 : _a.toString();
            }
            else if (request.url == '/rpc/theia_remote') {
                if (!request.body || !this.csrfToken || !this.clientVersion || !this.clientId) {
                    // TODO return an unhandled or something
                    baseOnProxyReq === null || baseOnProxyReq === void 0 ? void 0 : baseOnProxyReq(proxyReq, request, response);
                    return;
                }
                const body = request.body;
                delete request.body;
                if (!body.method || !body.params) {
                    this.logger.warn(`Improperly formatted theia_remote command. Body: ${body}`);
                    baseOnProxyReq === null || baseOnProxyReq === void 0 ? void 0 : baseOnProxyReq(proxyReq, request, response);
                    return;
                }
                const newBody = (() => {
                    switch (body.method) {
                        case "navigate_to_file":
                            const params = [`rstudioapi::navigateToFile('${body.params.file.replace("'", "\\'")}')`, ""];
                            if (this.rstudioCliContribution.isNewRStudio()) {
                                params.push(0);
                            }
                            return {
                                method: "console_input",
                                params: params,
                            };
                            break;
                    }
                })();
                if (!newBody) {
                    this.logger.warn(`Improperly formatted theia_remote command. Body: ${body}`);
                    return;
                }
                newBody.clientId = this.clientId;
                newBody.clientVersion = this.clientVersion;
                const bodyStringified = JSON.stringify(newBody);
                proxyReq.setHeader('Content-Length', bodyStringified.length);
                proxyReq.setHeader('X-CSRF-Token', this.csrfToken);
                proxyReq.path = '/rpc/console_input';
                proxyReq.write(bodyStringified);
                return;
            }
            baseOnProxyReq === null || baseOnProxyReq === void 0 ? void 0 : baseOnProxyReq(proxyReq, request, response);
        };
        return (0, http_proxy_middleware_1.createProxyMiddleware)(basePath, baseOptions);
    }
};
__decorate([
    (0, inversify_1.inject)(core_1.ILogger),
    __metadata("design:type", Object)
], RStudioServerProxyContribution.prototype, "logger", void 0);
__decorate([
    (0, inversify_1.inject)(server_proxy_url_manager_1.ServerProxyUrlManager),
    __metadata("design:type", server_proxy_url_manager_1.ServerProxyUrlManager)
], RStudioServerProxyContribution.prototype, "serverProxyUrlManager", void 0);
__decorate([
    (0, inversify_1.inject)(rstudio_cli_contribution_1.RStudioCliContribution),
    __metadata("design:type", rstudio_cli_contribution_1.RStudioCliContribution)
], RStudioServerProxyContribution.prototype, "rstudioCliContribution", void 0);
RStudioServerProxyContribution = __decorate([
    (0, inversify_1.injectable)(),
    __param(0, (0, inversify_1.inject)(RStudioServerProxyInstanceBuilder)),
    __metadata("design:paramtypes", [RStudioServerProxyInstanceBuilder])
], RStudioServerProxyContribution);
exports.RStudioServerProxyContribution = RStudioServerProxyContribution;
//# sourceMappingURL=rstudio-server-proxy-contribution.js.map